# COMPSHIFT - Set emission line centers, one per fiber, then compute mean
#             shift of Hectospec or Hectochelle images relative to standard
#
# December 17, 2003
# By Doug Mink, SAO Telescope Data Center

procedure compshift (images)

file	images=""	{prompt="Image files for which to find line centers"}
file	refimage=""	{prompt="Image file in which to find line centers"}
file	centers=""	{prompt="File containing nominal coordinates of lines"}
file	refcenters=""	{prompt="File containing reference line centers"}
bool	writeshifts=no	{prompt="Output file of apertures and x y shifts (y or n)"}
bool	oneshift=yes	{prompt="Compute single shift for all aps (y or n)"}
real	xshift=0.0	{prompt="Returned X shift in pixels"}
real	yshift=0.0	{prompt="Returned Y shift in pixels"}
bool	deltemp=yes	{prompt="Delete temporary files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	refim, temp1, temp2, listfile, filename, outfile
string	imagelist, temp
bool	vb
real	x, y
char	tchar
int	i, j

	refim = refimage
	temp1 = ""
	imagelist = images
	vb = verbose
	refc = refcenters
	x = 0.0
	y = 0.0

# Find emission line centers for the reference image from nominal positions
	center.coords = centers
	center.plotfile = ""
	center.datapars = "hectospec.datapars"
	center.centerpars = "hectospec.centerpars"
	center.interactive = no
	center.radplots = no
	center.verify = no
	center.icommands = ""
	center.gcommands = ""

# Find emission line centers for the reference image from nominal positions
	if (refc == "") {
	    temp1 = mktemp ("bluecent")
	    center.output = temp1
	    center (refim)

# Extract center coordinates for each fiber from center output file
	    refcenters = refim // ".blue"
	    refc = refcenters
	    temp =""
	    list = temp1
	    for (i = 0; i < 45; i=i+1) {
		j = fscan (list, temp)
#		print (temp)
		}
	    j = fscan (list, x, y)
	    j = fscan (list, x, y)
	    print (x," ",y,>refcenters)
#	    if (vb) {
#		print (x," ",y)
#		}
	    while (fscan (list, x, y) != EOF) {
		j = fscan (list, x, y)
		print (x," ",y,>>refcenters)
#		if (vb) {
#		    print (x," ",y)
#		    }
		}	
	    if (vb) {
		print ("COMPSHIFT: ",refcenters," made from ",refim)
		}
	    }
	center.coords = refc

#  Make list file of image files from input
        tchar = substr (imagelist,1,1)
        if (tchar != '@') {
            temp2 = mktemp ("cmpshft")
            print (imagelist,>temp2)
            listfile = temp2
            }
        else
            listfile = substr(imagelist,2,256)

# Fit centers for each image, one at a time
	list = listfile
	compshift1.verbose = vb
	compshift1.writeshifts = writeshifts
	compshift1.oneshift = oneshift
	compshift1.refcenters = refcenters
	while (fscan (list,filename) != EOF) {
	    compshift1 (filename,refc)

# write shifts into header of data file
	    if (oneshift) {
		xshift = compshift1.xshift
		yshift = compshift1.yshift
		arg = filename // " xshift=" // xshift // " yshift=" // yshift
		}
	    else {
		arg = filename // " shftfile=" // filename // ".shifts"
		}
	    sethead (arg)
	    }

	xshift = compshift1.xshift
	yshift = compshift1.yshift
	if (deltemp) {
	    if (temp1 != "") delete (temp1)
	    delete (temp2)
	    }
end

# Dec 15 2003	New program to comput X-Y shifts from a reference file
# Dec 16 2003	Drop WCSTools calls and do all file reading inline
# Dec 17 2003	Add switches to print shifts to file

# Jan 27 2004	Write shifts into header of data file
