# COMPSHIFT1 - Set emission line centers, one per fiber, then compute mean
#             shift of Hectospec or Hectochelle images relative to standard
# December 17, 2003
# By Doug Mink, SAO Telescope Data Center

procedure compshift1 (image, refcenters)

file	image=""	{prompt="Image files for which to find line centers"}
file	refcenters=""	{prompt="File containing reference line centers"}
bool	writeshifts=no	{prompt="Output file of apertures and x y shifts (y or n)"}
bool	oneshift=no	{prompt="Print same shift for all apertures (y or n)"}
real	xshift=0.0	{prompt="Returned X shift in pixels"}
real	yshift=0.0	{prompt="Returned Y shift in pixels"}
bool	verbose=no	{prompt="Print process progress (y or n)"}

string *reflist

begin

file	filename, centout, fileout
string	line
bool	dellist, vb, perap
real	dx,dy,sumxdiff,sumydiff,dn,x,y,x0,y0,xdiff,ydiff
real	maxxdiff,minxdiff,maxydiff,minydiff
int	i, j, id

	filename = image
	if (writeshifts) {
	    if (oneshift) {
		fileout = filename // ".shift"
		}
	    else {
		fileout = filename // ".shifts"
		}
	    }
	else {
	    fileout = ""
	    }
	vb = verbose
	perap = !oneshift
	refc = refcenters
	x = 0.0
	y = 0.0
	x0 = 0.0
	y0 = 0.0

# Find emission line centers for the reference image from nominal positions
	center.coords = refc
	center.plotfile = ""
	center.datapars = "hectospec.datapars"
	center.centerpars = "hectospec.centerpars"
	center.interactive = no
	center.verify = no
	center.radplots = no
	center.icommands = ""
	center.gcommands = ""
	center.flpar = no
	center.mode = "ql"

# Fit centers for one image
	if (vb) {
	    print ("COMPSHIFT: Finding centers in ",filename)
	    }
	centout = filename // ".blue"
	center.output = centout
	center (filename)

# Extract center coordinates for each fiber from center output file
	if (vb) {
	    print ("COMPSHIFT: Reading reference centers from ",refc)
	    }
	dn = 0.0
	id = 0
	sumxdiff = 0.0
	sumydiff = 0.0
	maxxdiff = 999.0
	list = centout
	for (i = 0; i < 45; i=i+1) {
	    j = fscan (list, line)
	    }
	reflist = refc
	while (fscan (list, x, y) != EOF) {
	    i = fscan (list, x, y)
	    j = fscan (reflist, x0, y0)
	    xdiff = x0 - x
	    ydiff = y0 - y
	    if (maxxdiff == 999.0) {
		maxxdiff = xdiff
		minxdiff = xdiff
		maxydiff = ydiff
		minydiff = ydiff
		}
	    if (xdiff > maxxdiff) maxxdiff = xdiff
	    if (xdiff < minxdiff) minxdiff = xdiff
	    if (ydiff > maxydiff) maxydiff = ydiff
	    if (ydiff < minydiff) minydiff = ydiff
	    sumxdiff = sumxdiff + xdiff
	    sumydiff = sumydiff + ydiff
	    dn = dn + 1.0
	    id = id + 1
	    if (fileout != "" && perap) {
		if (id == 1) {
		    printf ("%3d %.3f %.3f\n",id,xdiff,ydiff,> fileout)
		    }
		else {
		    printf ("%3d %.3f %.3f\n",id,xdiff,ydiff,>> fileout)
		    }
#		if (vb) {
#		    printf ("%3d: x= %.3f - %.3f = %.3f, y = %.3f - %.3f = %.3f\n",id,x,x0,xdiff,y,y0,ydiff)
#		    }
		}
	    }
	dx = sumxdiff / dn
	dy = sumydiff / dn
	xshift = dx
	yshift = dy

# Print same shifts for every aperture
	if (fileout != "" && !perap) {
	    printf ("  0 %.3f %.3f\n",xdiff,ydiff, > fileout)
	    }
	if (vb) {
	    printf ("%s: dx= %.3f (%.3f - %.3f) dy= %.3f (%.3f - %.3f)\n",filename,dx,minxdiff,maxxdiff,dy,minydiff,maxydiff)
	    }
end

# Dec 15 2003	New program to comput X-Y shifts from a reference file
# Dec 16 2003	Drop WCSTools calls and do all file reading inline
# Dec 17 2003	Add output file formatted as input for shiftap task
