# Run the 4 seperate gain corrections for HECTOSPEC
#
# Bill Wyatt, 2003-10-27 et. seq.
#
# Input may be a comma-separated list or a filename (with prefaced "@")
# If output list not given, the updates are in place, else the output
# list must match the input in number.
#

procedure gaincorr (input, output)

string input = ""        {prompt="Images to correct"}
string output= ""        {prompt="Output image"}

real   g1    = 1.0       {prompt="Gain corr. for amplifier 1"}
real   g2    = 1.0       {prompt="Gain corr. for amplifier 2"}
real   g3    = 1.03      {prompt="Gain corr. for amplifier 3"}
real   g4    = 1.0558    {prompt="Gain corr. for amplifier 4"}

bool   hupdate = yes     {prompt="Check header for GAINCORR and record same"}
bool   vb      = yes     {prompt="verbose output (y or n)"}

struct *infile
struct *outfile

begin

   string inlist, outlist
   string tchar
   string gcsect1, gcsect2, gcsect3, gcsect4
   string procid
   string fname
   file   imagei, imageo

   inlist  = input
   outlist = output
   
# see if output is null ==> gain correct file(s) in place

   procid = mktemp("tmp$")   # really the procid plus a character

   tchar = substr(inlist, 1, 1)
   if(tchar == "@") {
      infile = substr(inlist,2,100)
   }
   else {
     fname = procid // "gcin"
     infile = fname
     files(inlist, >fname)
   }

   tchar = substr(outlist, 1, 1)
   fname = procid // "gcout"
   outfile = fname
   if(tchar == "@") {
      outfile = substr(outlist,2,100)
   }
   else if (outlist == "") {
     files(inlist, >fname)
   } else {
     files(outlist, >fname)
   }    

   while( fscan(infile, imagei) != EOF) {
     if(fscan(outfile, imageo) == EOF) {
        printf("GAINCORR: output list too short (input=%s)?\n", imagei)
        break
     }
     if(vb) printf("GAINCORR: imarith on extns of %s to %s\n",
          str(imagei), str(imageo))

  # check if GAINCORRection has already been done
     fname = ""
     hedit(imagei // "[1]", "GAINCORR", ".") | scanf("%s", fname)

  # or, if HUPDATE is false, do it anyway, but no recording of header
     if(fname != "" && hupdate) {
       printf("skipping %s: already gain correctd\n", imagei)

       # if output is different, copy it over
       if(imagei != imageo) {
         if(access(imageo)) { 
             imdelete(imageo) 
         } else if(access(imageo // ".fits")) {
             imdelete(imageo // ".fits") 
         } else if(access(imageo // ".fit")) {
             imdelete(imageo // ".fit") 
         } else if(access(imageo // ".fts")) {
             imdelete(imageo // ".fts") 
         } else if(access(imageo // ".imh")) {
             imdelete(imageo // ".imh") 
         }
         msccmd("imcopy $input $output verbose=" // vb, imagei, imageo)
       }
       next  # skip to beginning of loop
     }
  #otherwise, do the corrections
     imcopy (imagei // "[0]", procid // "GC_0", verbose=vb)

     gcsect1 = procid // "GC_1"
     gcsect2 = procid // "GC_2"
     gcsect3 = procid // "GC_3"
     gcsect4 = procid // "GC_4"

     imarith(imagei // "[1]", '*', g1, gcsect1 ) # , verbose=vb)
     imarith(imagei // "[2]", '*', g2, gcsect2 ) # , verbose=vb)
     imarith(imagei // "[3]", '*', g3, gcsect3 ) # , verbose=vb)
     imarith(imagei // "[4]", '*', g4, gcsect4 ) # , verbose=vb)

  # Add GAINCORR keyword to header
     if(hupdate) {
       hedit.show = no
       hedit(gcsect1, "GAINCORR", (g1), add=yes, verify=no)
       hedit(gcsect2, "GAINCORR", (g2), add=yes, verify=no)
       hedit(gcsect3, "GAINCORR", (g3), add=yes, verify=no)
       hedit(gcsect4, "GAINCORR", (g4), add=yes, verify=no)
     }

     if(access(imageo)) { 
       imdelete(imageo) 
     } else if(access(imageo // ".fits")) {
       imdelete(imageo // ".fits") 
     } else if(access(imageo // ".fit")) {
       imdelete(imageo // ".fit") 
     } else if(access(imageo // ".fts")) {
       imdelete(imageo // ".fts") 
     } else if(access(imageo // ".imh")) {
       imdelete(imageo // ".imh") 
     }
     if(vb) print("GAINCORR: joining ",imageo)
  # assemble the output file
     mscjoin(procid // "GC", output = imageo, delete=yes, verbose=vb)
   }

   delete(procid // "*")

end

