# HCAL - Split multispec hectospec file in two, identify, reidentify, reassemble
# June 10, 2004
# By Doug Mink, SAO Telescope Data Center

procedure hcal (calfile)

file	calfile=""	{prompt="Calibration file name"}
int	apodd=89	{prompt="Odd aperture to calibrate"}
int	apeven=88	{prompt="Even aperture to calibrate"}
file	oldcal=""	{prompt="Multispec file for previously calibrated spectrum"}
file	olddir=""	{prompt="Directory for previously calibrated spectrum"}
file	coordlist="idhenear.dat"	{prompt="User coordinate list"}
string	function="legendre"	{prompt="Coordinate function"}
int	order=7		{prompt="Order of coordinate function"}
bool	saveold=no	{prompt="Save unprocessed file to Nodisp/ (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	caleven, calodd, calall, eventemp, oddtemp, calms, idall, clist, oldid
file	calref, idodd, ideven, caldisp, calndisp, oldpath,oldref,newdb,newfile
string	arg, oddarg, evenarg, imsect, oddaps, evenaps, calsect, calline
string	func
bool	vb
int	lineodd, lineven, ord, lroot

	vb = verbose
	clist = coordlist
	calall = calfile
	smatch (calall, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    calall = substr (calall, 1, lroot)
	    }
	calms = calall // ".ms.fits"
	if (!access (calms)) {
	    print ("HCAL: ", calms, " is not accessible")
	    return
	    }

# If calibration has not been done, run identify for even and odd spectra
	if (oldcal == "") {
	    oddtemp = calall // ".oddlist"
	    eventemp = calall // ".evenlist"
	    calodd = calall // "odd.ms.fits"
	    caleven = calall // "even.ms.fits"
	    idodd = "database/id" // calall // "odd.ms"
	    ideven = "database/id" // calall // "even.ms"
	    clist = coordlist
	    func = function
	    ord = order

# Make file of only odd apertures
	    if (access (calodd)) {
		if (vb) {
		    print ("HCAL: Odd apertures already written to ", calodd)
		    }
		}
	    else {
		scopy (calms, calodd, apertures="1x2")
		if (vb) {
		    print ("HCAL: Odd apertures written to ", calodd)
		    }
		}

# Make file of only even apertures
	    if (access (caleven)) {
		if (vb) {
		    print ("HCAL: Even apertures already written to ", caleven)
		    }
		}
	    else {
		scopy (calms, caleven, apertures="2x2")
		if (vb) {
		    print ("HCAL: Even apertures written to ", caleven)
		    }
		}

# Identify one odd aperture
	    lineodd = (apodd / 2) + 1
	    calline = "line " // lineodd
	    identify.section=calline
	    identify.coordlist=clist
	    identify.function=func
	    identify.order=ord
	    if (!access (idodd)) {
		if (vb) {
		    print ("HCAL: identify ",calodd)
		    }
		identify (calodd)
		}
	    else if (vb) {
		print ("HCAL: ", calodd, " already identified")
		}

# Reidentify rest of odd apertures
	    reidentify.interactive = "yes"
	    reidentify.section = "middle line"
	    reidentify.newaps = yes
	    reidentify.refit = yes
	    reidentify.trace = yes
	    reidentify.override = no
	    reidentify.step = "1"
	    reidentify.nsum = "10"
	    reidentify.nlost = 20
	    reidentify.cradius = 5.
	    reidentify.threshold = 10.
	    reidentify.addfeatures = no
	    reidentify.maxfeatures = 40
	    reidentify.answer = "YES"
	    reidentify.coordlist = clist
	
	    calsect = "[*," // lineodd // "]"
	    calref = calodd // calsect
#	    reidentify.section = calsect
	    if (vb) {
		print ("HCAL: reidentify ",calref," ",calodd)
		}
	    reidentify (calref, calodd)

# Identify one even aperture
	    lineven = apeven / 2
	    identify.section = "line " // lineven
	    if (!access (ideven)) {
		if (vb) {
		    print ("HCAL: identify ",caleven)
		    }
		identify (caleven)
		}
	    else if (vb) {
		print ("HCAL: ", caleven, " already identified")
		}

# Reidentify rest of even apertures
	    calsect = "[*," // lineven // "]"
	    calref = caleven // calsect
#	    reidentify.section = calsect
	    calref = caleven // calsect
	    if (vb) {
		print ("HCAL: reidentify ",calref," ",caleven)
		}
	    reidentify (calref, caleven)

# Combine two sets of dispersion functions into one
	    calref = calall // ".ms"
	    idall = "database/id" // calref
	    arg = "-e s/hearodd/hear/g " // idodd
	    sed (arg, >idall)
	    arg = "-e s/heareven/hear/g " // ideven
	    sed (arg, >>idall)
	    if (vb) {
		print ("HCAL: ", ideven, " + ", idodd, " => ", idall)
		}
	    }

# Use previously calibrated spectrum as a reference
	else {

# Move over old calibration file, line list, and id file
	    smatch (oldcal, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		oldref = substr (oldcal, 1, lroot)
		}
	    else {
		oldref = oldcal
		oldcal = oldcal // ".fits"
		}
	    if (olddir != "") {
		if (!access ("database/")) {
		    mkdir ("database")
		    }
		newdb = "database/id" // oldref
		if (!access (newdb)) {
		    oldpath = olddir // "/database/id" // oldref
		    if (vb) {
			print ("HCAL: database file id",oldref," copied.")
			}
		    copy (oldpath, "database/")
		    }
		if (!access (oldcal)) {
		    oldpath = olddir // "/" // oldcal
		    if (vb) {
			print ("HCAL: calibration spectrum ",oldcal," copied.")
			}
		    imcopy (oldpath, ".")
		    }
		if (!access (clist)) {
		    oldpath = olddir // "/" // clist
		    if (vb) {
			print ("HCAL: calibration line list ",clist," copied.")
			}
		    copy (oldpath, ".")
		    }
		}
	    if (vb) {
		print ("HCAL: Running reidentify with ",oldcal)
		}

	    reidentify.interactive = "yes"
	    reidentify.section = "middle line"
	    reidentify.newaps = yes
	    reidentify.refit = yes
	    reidentify.trace = yes
	    reidentify.override = no
	    reidentify.step = "1"
	    reidentify.nsum = "10"
	    reidentify.nlost = 20
	    reidentify.cradius = 5.
	    reidentify.threshold = 10.
	    reidentify.addfeatures = no
	    reidentify.maxfeatures = 40
	    reidentify.answer = "YES"
	    reidentify.coordlist = clist
	    reidentify (oldref, calms)
	    calref = calall // ".ms"
	    }

# Dispersion-correct file
	if (vb) {
	    print ("HCAL: Set REFSPEC1 in ",calms," to ",calref)
	    }
	hedit (calms, "REFSPEC1", calref, verify=no, show=vb, update=yes)
#	refspec (calms, references=calref)
	caldisp = calall // ".disp.ms.fits"
	dispcor (calms, caldisp, linearize=no)
	if (saveold) {
	    if (!access ("Nodisp/")) {
		mkdir ("Nodisp")
		}
	    newfile = "Nodisp/" // calms
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    move (calms, "Nodisp")
	    if (vb) {
		print ("HCAL: Original ", calms, " => Nodisp/")
		}
	    }
	else {
	    imdelete (calms)
	    }
	rename (caldisp, calms)
end

# Mar 31 2004	New script
# Jun  3 2004	Process input file name; run dispcor on file
# Jun 10 2004	Make saving original file optional
