# HCOMBINE - Combine merged images, removing cosmic rays in the process
# June 16, 2004
# By Doug Mink, SAO Telescope Data Center

procedure hcombine (images, output)

string	images=""	{prompt="Images to combine"}
file	output=""	{prompt="Output image"}
bool	fix = yes	{prompt="Replace cosmic rays using fixpix (y or n)"}
string	combine = "average"	{prompt="Combine images (average|median|sum)"}
string	reject = "none"	{prompt="Reject method (none|minmax|ccdclip|crreject|sigclip|avsigclip|pclip)"}
int	threshold = 4	{prompt="N sigma threshold for cosmic ray difference"}
bool	deltemp = no	{prompt="Delete intermediate image files (y or n)"}
bool	verbose = yes	{prompt="Print process progress (y or n)"}

begin

int	ifile, nfiles, icom, lname, tval, lbrack, rbrack
string	iml, tchar, imx, iml1, imagelist, cleanlist, imageroot
string	outroot, extxt, combim, rejim
file	tempmed, tempdiff, tempbad, tempsig, imagei, listfile, imageout, imfile
real	medmed, medi, factor
bool	dellist, vb, fixim

	vb = verbose
	dellist = yes
	imagelist = images
	imageout = output
	imx = ".fits"
	rejim = reject
	combim = combine
	tval = threshold
	fixim = fix
	smatch (imageout, ".fits")
	if (smatch.found) {
	    outroot = substr (imageout,1,smatch.start-1)
	    }
	else {
	    outroot = imageout
	    }
#	if (vb) {
#	    print ("HCOMBINE: output file ",imageout," root is ",outroot)
#	    }

# Initialize imcombine
	imcombine.headers = ""
	imcombine.bpmasks = ""
	imcombine.nrejmasks = ""
	imcombine.expmasks = ""
	imcombine.sigmas = ""
	imcombine.logfile = "STDOUT"
	imcombine.project = no
	imcombine.outlimits = ""
	imcombine.offsets = "none"
	imcombine.masktype = "none"
	imcombine.maskvalue = 0.
	imcombine.blank = 0.
	imcombine.scale = "none"
	imcombine.zero = "none"
	imcombine.weight = "none"
	imcombine.statsec = ""
	imcombine.expname = "EXPTIME"
	imcombine.lthreshold = INDEF
	imcombine.hthreshold = INDEF
	imcombine.nlow = 34
	imcombine.nhigh = 0
	imcombine.nkeep = 0
	imcombine.mclip = no
	imcombine.lsigma = 3.
	imcombine.hsigma = 3.
	imcombine.rdnoise = "0."
	imcombine.gain = "1."
	imcombine.snoise = "0."
	imcombine.sigscale = 0.1
	imcombine.pclip = -0.5
	imcombine.grow = 0.

#  Make median from input images
#	tempmed = mktemp ("zapmed") // ".fits"
	tempmed = outroot // "rmed.fits"
	imcombine.combine = "median"
	imcombine.reject = "crreject"
	imcombine.outtype="real"
	imcombine.rejmasks = outroot // "rej"
	if (vb) {
	    print ("HCOMBINE: Median image ",tempmed," from ",imagelist)
	    }
	imcombine (imagelist, tempmed)

#  Find median value of combined image
	istat (tempmed)
	medmed = istat.median
	if (vb) {
	    print ("HCOMBINE: Median pixel value of ",tempmed," is ",medmed)
	    }

#  Make standard deviation file from median file
	tempsig = mktemp ("zapsig") // ".fits"
	expression = nsig // "*sqrt(a)"
	imexpr (expression, tempsig, tempmed)
	if (vb) {
	    print ("HCOMBINE: ", nsig, " sigma error file is ",tempsig)
	    }

#  Make one-per-line list of files from input string or file of filenames

#  If first character of imagelist is @, get number of images in list
        tchar = substr (imagelist,1,1)
        icom = stridx (',',imagelist)
        if (tchar == "@") {
            listfile = substr (imagelist,2,100)
            dellist = no
            }

#  If comma-delimited list, list to file, adding image header extension
        else {
            listfile = mktemp ("zaplist")
            files (imagelist,>listfile)
            dellist = yes
            }
        if (vb)
            print ("HCOMBINE: Reading filenames from ",listfile)

#  Remove cosmic rays from one image at a time
	cleanlist = ""
        list = listfile
        while (fscan (list, imagei) != EOF) {

#  Find root of filename (without .fits, with _extname)
#  Add _extnum, if present
	    smatch (imagei, "[")
	    if (smatch.found) {
		lbrack = smatch.start
		}
	    else {
		lbrack = 0
		}
	    smatch (imagei, "]")
	    if (smatch.found) {
		rbrack = smatch.start
		}
	    else {
		rbrack = 0
		}
	    if (lbrack > 0 && rbrack > 0) {
		ext = substr (imagei,lbrack+1,rbrack-1)
		}
	    smatch (imagei, ".fits")
	    if (smatch.found) {
		imageroot = substr (imagei,1,smatch.start-1)
		if (lbrack > 0 && rbrack > 0) {
		    imageroot = imageroot // "_" // ext
		    }
		}
	    else {
		if (lbrack > 0 && rbrack > 0) {
		    imageroot = imagei // "_" // ext
		    imagei = imagei // ".fits" // "[" // ext // "]"
		    }
		else {
		    imageroot = imagei
		    imagei = imagei // ".fits"
		    }
		}

#	Compute ratio of this image to the median image
	    istat (imagei)
	    medi = istat.median
	    factor = medmed / medi
	    if (vb) {
		print ("HCOMBINE: median of ",imagei," is ",medi)
		}

#	Make a bad pixel map for this image
            tempdiff = imageroot // "diff.fits"
	    imarith (imagei,'*',factor,tempdiff)
	    imarith (tempdiff,'-', tempmed, tempdiff)
	    if (vb) {
		print ("HCOMBINE: Scaled difference file ", tempdiff)
		}

#	All pixels with value less than or equal to threshold will be 0 (good)
#	All pixels with value greater than threshold will be 1 (bad)
            tempbad = imageroot // "bad.fits"
	    imarith (tempdiff,'-', tempsig, tempbad)
	    imreplace (tempbad, 0, upper=0)
	    imreplace (tempbad, 1, lower=1)
	    if (vb) {
		print ("HCOMBINE: Bad pixel file ", tempbad)
		}

#	Make temporary file to add so original image files are unchanged
            imfilee= mktemp ("zapclean")

#	Replace cosmic rays with data around them if fix is set
	    if (fixim) {
		imfile = imageroot // "c.fits"
		imcopy (imagei, imfile)
		fixpix (imfile, tempbad)
		if (vb) {
		    print ("HCOMBINE: Cleaned image ", imfile)
		    }
		if (deltemp)
		    imdelete (tempbad)
		}
	    else {
		imfile = imageroot // "t.fits"
		imcopy (imagei, imfile)
		hedit (imfile,"BPM",tempbad,addonly=yes,update=yes,verify=no,show=no)
		if (vb) {
		    print ("HCOMBINE: New BPM value in ", imfile)
		    }
		}
	    if (cleanlist == "") {
		cleanlist = imfile
		}
	    else {
		cleanlist = cleanlist // "," // imfile
		}
	    }

#	Sum fixed images
	imcombine.combine = combim
	imcombine.reject = rejim
	if (vb) {
	    print ("HCOMBINE: Combined file ", imageout)
	    }
	imcombine.outtype="real"
	imcombine.rejmasks = ""
	imcombine (cleanlist, imageout)
	if (dellist) {
	    delete (listfile)
	    }
	if (deltemp) {
	    imdelete (tempmed)
	    delete (cleanlist)
	    }
end

# Oct  1 2003	New program to remove cosmic rays from spectra
# Oct  9 2003	Add threshold parameter and use temp files
# Oct 24 2003	Initialize cleanlist concatenation
# Oct 28 2003	Add options to add files with masks
# Oct 31 2003	Add combination and rejection option parameters

# Jun 16 2004	Change name fromZAP to HCOMBINE
# Jun 16 2004	Compute per pixel n*sigma from median file
