# HEXTRACT -- Extract  Hectospec image into multispec file
# By Doug Mink, SAO Telescope Data Center
# June 16, 2004

procedure hextract (input, mapfile)

file	input = ""	{prompt="Input image to extract"}
file	mapfile=""	{prompt="Fiber to sky map file"}

file	apref = ""	{prompt="Aperture reference spectrum"}
bool	resize=no	{prompt="Resize object apertures?"}
bool	recenter=no	{prompt="Recenter object apertures?"}
bool    shift = yes	{prompt="Use average shift instead of recentering?"}
real    npeaks = 0.2	{prompt="Select brightest peaks"}
bool	verbose = yes	{prompt="Log progress (y or n)"}

begin
	bool vb
	file imin, imout, imroot, imref
	int lroot
	string history, map

	imin = input
	imref = apref
	vb = verbose
	map = mapfile

# Remove excess space from map file so its lines fit on header lines
	if (map != "") {
	    hsmapfix (map)
	    }

# Make output file name
	smatch (imin, ".fits")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imout = substr (imin, 1, lroot) // ".ms.fits"
	    }
	else {
	    imout = imin // ".ms.fits"
	    imin = imin // ".fits"
	    }
	if (vb) {
	    print ("HNORM: ", imin," -> ",imout," using ", imref," and ",map)
	    }

# Set up dohspec parameters
	dohspec.resize = resize
	dohspec.apidtable = map
	dohspec.apref = imref
	dohspec.flat = ""
	dohspec.throughput = ""
	dohspec.arcs1 = ""
	dohspec.arcs2 = ""
	dohspec.arcreplace = ""
	dohspec.arctable = ""
	dohspec.readnoise = "1"
	dohspec.gain = "1"
	dohspec.datamax = INDEF
	dohspec.fibers = 300
	dohspec.width = 8.
	dohspec.minsep = 5.
	dohspec.maxsep = 100.
	dohspec.crval = "INDEF"
	dohspec.cdelt = "INDEF"
	dohspec.objaps = ""
	dohspec.skyaps = ""
	dohspec.arcaps = ""
	dohspec.objbeams = "1"
	dohspec.skybeams = "0"
	dohspec.arcbeams = ""
	dohspec.scattered = no
	dohspec.fitflat = no
	dohspec.clean = no
	dohspec.dispcor = no
	dohspec.savearcs = no
	dohspec.skyalign = no
	dohspec.skysubtract = no
	dohspec.skyedit = no
	dohspec.saveskys = no
	dohspec.splot = yes
	dohspec.redo = no
	dohspec.update = no
	dohspec.batch = no
	dohspec.listonly = no
	dohspec.shift = shift
	dohspec.npeaks = npeaks
	dohspec.params = ""

# Extract spectra to multispec file using dohspec
	dohspec (imin)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " extracted from " // imin
	hedit (imout, "HEXTRACT", history, add+, verify-)
	hedit (imout, "HEXAPREF", imref, add+, verify-)
	hedit (imout, "HEXAPMAP", map, add+, verify-)

end

# Jun 15 2004	Add recenter, shift, npeaks, parameters for dohspec and proc
# Jun 16 2004	Add resize parameter for dohspec and apscript
