# HLIN -- Dispersion correct and linearize spectrum, placing input in Nodisp/
# By Doug Mink, SAO Telescope Date Center
# June 10, 2004

procedure hlin (input)

string	input = ""	{prompt="Input multispec image to correct"}
string	calfile=""	{prompt="Calibration file for dispersion correction"}
real wl1=3700.0		{prompt="Starting wavelength for output spectra (A)"}
real wl2=9150.0		{prompt="Ending wavelength for output spectra (A)"}
real dwl=1.21	{prompt="Delta wavelength per pixel for output spectra (A)"}
int npix = INDEF	{prompt="Number of pixels per output spectra"}
bool deltemp = no	{prompt="Delete temporary files (y or n)"}
bool saveold = yes	{prompt="Save original files in Nodisp/ and Nonlin/ (y or n)"}
bool verbose = yes	{prompt="Log progress (y or n)"}
bool debug = no		{prompt="Too much info from sumspec (y or n)"}

begin
	bool vb, db, save
	file imin, imlin, imdisp, imroot, imcal, newfile
	int lroot, lref

	imin = input
	imcal = calfile
	vb = verbose
	db = debug
	save = saveold

# Remove extension from calibration file, if one is present
	smatch (imcal, ".fits")
	if (smatch.found) {
	    lref = smatch.start - 1
	    imcal = substr (imcal, 1, lref)
	    }

# Make output file name and check to make sure input is multispec
	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imroot = substr (imin, 1, lroot)
	    }
	else {
	    print ("HLIN: ", imin," is not a multispec file")
	    return
	    }

# Make sure filename ends in .fits
	smatch (imin, ".fits")
	if (!smatch.found) {
	    imin = imin // ".fits"
	    }

# Dispersion-correct file
	if (vb) {
	    print ("HLIN: Set REFSPEC1 in ",imin," to ",imcal)
	    }
	hedit (imin, "REFSPEC1", imcal, verify=no,show=vb,add=yes,update=yes)
#	refspec (imin, references=imcal)
	imdisp = imroot // ".disp.ms.fits"
	dispcor (imin, imdisp, linearize=no)

# Save original file, if requested
	if (save) {
	    if (!access ("Nodisp/")) {
		mkdir ("Nodisp")
		}
	    newfile = "Nodisp/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    move (imin, "Nodisp")
	    if (vb) {
		print ("HLIN: Original ", imin, " => Nodisp/")
		}
	    }
	else {
	    delete (imin)
	    }
	rename (imdisp, imin)

# Make output file name
	imlin = imroot // ".lin.ms.fits"

# Rebin spectra identically
	rvsao (motd=no)
	sumspec.spectra = imin
	sumspec.compfile = imlin
	sumspec.specnum = "1-300"
	sumspec.specband = 0
	sumspec.specdir = ""
	sumspec.compname = ""
	sumspec.compdir = ""
	sumspec.nspec = 300
	sumspec.save_names = no
	sumspec.normin = 0.
	sumspec.fixbad = "no"
	sumspec.cont_remove = "no"
	sumspec.reject = no
	sumspec.abs_reject = 100.
	sumspec.em_reject = 2.
	sumspec.contout = "no"
	sumspec.cont_plot = no
	sumspec.cont_add = 0.
	sumspec.spec_smooth = 0
	sumspec.st_lambda = wl1
	sumspec.end_lambda = wl2
	sumspec.pix_lambda = dwl
	sumspec.npts = npix
	sumspec.complog = no
	sumspec.interp_mode = "spline3"
	sumspec.normout = 0.
	sumspec.spec_plot = no
	sumspec.spec_int = no
	sumspec.comp_plot = vb
	sumspec.comp_int = no
	sumspec.velcomp = INDEF
	sumspec.zcomp = INDEF
	sumspec.svel_corr = "none"
	sumspec.nsmooth = 0
	sumspec.device = "stdgraph"
	sumspec.plotter = "stdplot"
	sumspec.logfiles = "STDOUT,displin.log"
	sumspec.nsum = 1
	sumspec.debug = db
	sumspec.cursor = ""
	sumspec (imin, imlin)

# Move original file to Nonlin directory, if requested
	if (save) {
	    if (!access ("Nonlin/")) {
		mkdir ("Nonlin")
		}
	    newfile = "Nonlin/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    move (imin, "Nonlin")
	    if (vb) {
		print ("HLIN: Nonlinear ", imin, " => Nonlin/")
		}
	    }
	else {
	    delete (imin)
	    }

# Rename result file to original file name
	rename (imlin, imin)

end
