# HMERGE - Combine 4 amps on 2 chips into one Hectospec or Hectochelle image
# June 9, 2004
# By Doug Mink, SAO Telescope Data Center

procedure hmerge (images)

string	images=""	{prompt="Images to merge"}

bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, temp1, temp2, temp3, temp4, im4, imin, templist, imout
string	amp1, amp2, amp3, amp4, arg1, imx, ims, history
bool	dellist, vb

	vb = verbose
	imx = ".fits"
	ims = image

	if (ims == "") {
	    print ("HMERGE: Combine 4 amplifiers on chips from one Hectospec or Hectochelle image")
	    print ("Usage: mergechips <input image with 4 extensions> <output image>")
	    return
	    }

# Make directory in which to save processed multi-extension files
        if (!access ("Unmerged"))
            mkdir ("Unmerged")

#  Make temporary list of input files
	templist = mktemp ("mergelist")
	files (ims, >templist)
	list = templist
        while (fscan (list,imin) != EOF) {

# Find root name of image
	    imroot = imin
	    smatch (imroot, imx)
	    if (smatch.found) {
		imroot = substr (imroot,1,smatch.start-1)
		}
	    else {
		imroot = imageout
		}
	    if (vb) {
		print ("HMERGE: image root is ",imroot)
		}

#  Split out one image per amplifier
	    temp1 = mktemp ("hamp1") // imx
	    amp1 = imin // "[1]"
	    imcopy (amp1, temp1)
	    temp2 = mktemp ("hamp2") // imx
	    amp2 = imin // "[2]"
	    imcopy (amp2, temp2)
	    temp3 = mktemp ("hamp3") // imx
	    amp3 = imin // "[3]"
	    imcopy (amp3, temp3)
	    temp4 = mktemp ("hamp4") // imx
	    amp4 = imin // "[4]"
	    imcopy (amp4, temp4)

#  Rename original file
	    im4 = "Unmerged/" // imin
	    rename (imin, im4)

#  Join two chips into a single image
	    arg1 = temp1 // "," // temp2 // "[-*,*]," // temp3 // "," // temp4 // "[-*,*]"
	    imjoin (arg1, imin, 1)

	

# Make history line including time for preamp correction
            gettime ()
	    history = gettime.date // " " // gettime.time // " 4 amplifiers merged into one image"
	    hedit (imin, "HMERGE", history, add+, verify-)

	    if (deltemp) {
		imdelete (temp1)
		imdelete (temp2)
		imdelete (temp3)
		imdelete (temp4)
		}
	    }
	if (deltemp) {
	    delete (templist)
	    }
end

# Dec  8 2003	New program to make a single image from two chip images

# May 19 2004	Rename from HCOMBINE to MERGECHIPS and include amp combine
# May 20 2004	Rewrite to process multiple files at a time
# May 26 2004	Fix bug: use delete instead of imdelete for templist
# Jun  3 2004	Change name to MERGEAMPS; fix amp1 temporary file name
# June 4 2004	Add history in MERGEAMP keyword; change Amps/ to Unmerged/
# Jun  9 2004	Change name from MERGEAMPS to HMERGE
