# HNORM -- Divide extracted multispec spectra by pixel normalization file
#          saving original file in Nonorm/
# By Doug Mink, SAO Telescope Data Center
# June 16, 2004

procedure hnorm (input)

string	input = ""	{prompt="Input multispec image to correct"}
string	normfile=""	{prompt="Pixel to pixel normalization file from makenorm"}
bool deltemp = no	{prompt="Delete temporary files (y or n)"}
bool saveold = yes	{prompt="Save unprocessed file in Nonorm/ (y or n)"}
bool verbose = yes	{prompt="Log progress (y or n)"}

begin
	bool vb
	file imin, imout, imroot, imnorm, newfile
	int lroot
	string history

	imin = input
	vb = verbose
	imnorm = normfile

# Make output file name and check to make sure input is multispec
	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imout = substr (imin, 1, lroot) // ".norm.ms.fits"
	    }
	else {
	    print ("HNORM: ", imin," is not a multispec file")
	    return
	    }

# Make sure filename ends in .fits
	smatch (imin, ".fits")
	if (!smatch.found) {
	    imin = imin // ".fits"
	    }

# Divide out normalization spectra
	imarith (imin, "/", imnorm, imout)

# Move original file to Nonorm directory
	if (saveold) {
	    if (!access ("Nonorm/")) {
		mkdir ("Nonorm")
		}
	    newfile = "Nonorm/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    move (imin, "Nonorm")
	    if (vb) {
		print ("HNORM1: Original ", imin, " => Nonorm/")
		}
	    }
	else {
	    delete (imin)
	    }

# Rename result file to original file name
	rename (imout, imin)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " pixels normalized by " // imnorm
	hedit (imin, "PIXNORM", history, add+, verify-)

end
# Jun 16 2004	Change name from HNORM1 to HNORM
