# HPROC - Run image processing tasks one amp at a time
# June 8, 2004
# By Doug Mink, SAO Telescope Data Center

procedure procamps (images)

string	images=""	{prompt="Images to merge"}

bool	fixpreamp=yes	{prompt="Run preampfix to clean overscan (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, temp1, temp2, temp3, im2, im4, imin, templist
string	chip1, chip2, arg1, imx, ims, imsec, history
bool	dellist, vb

	vb = verbose
	imx = ".fits"
	ims = image

	if (ims == "") {
	    print ("HPROC: Run image processing tasks one amp at a time")
	    print ("Usage: procamp <input images with 4 extensions>")
	    return
	    }

# Make directory in which to save raw multi-extension files
	if (!access ("Raw"))
	    mkdir ("Raw")

# Make directory in which to save preampfixed multi-extension files
	if (!access ("Unproc"))
	    mkdir ("Unproc")

#  Make temporary list of input files
	templist = mktemp ("proclist")
	files (ims, >templist)
	list = templist
        while (fscan (list,imin) != EOF) {

# Find root name of image
	    imroot = imin
	    smatch (imroot, imx)
	    if (smatch.found) {
		imroot = substr (imroot,1,smatch.start-1)
		}
	    else {
		imroot = imageout
		}
#	    if (vb) {
#		print ("HPROC: image root is ",imroot)
#		}

# Fix preamp output using Brian McLeod's task
	    temp1 = mktemp (imroot) // "p.fits"
	    if (verbose)
		print ("HPROC: Making preamp correction for ",imin)
	    preampfix (imin, temp1)
	    if (verbose)
		print ""

# Make history line including time for preamp correction
            gettime ()
	    history = gettime.date // " " // gettime.time // " preamplifier correction made"
	    imsec = temp1 // "[1]"
	    hedit (imsec, "PREAMPFX", history, add+, verify-)
	    imsec = temp1 // "[2]"
	    hedit (imsec, "PREAMPFX", history, add+, verify-)
	    imsec = temp1 // "[3]"
	    hedit (imsec, "PREAMPFX", history, add+, verify-)
	    imsec = temp1 // "[4]"
	    hedit (imsec, "PREAMPFX", history, add+, verify-)

#  Save preampfixed file to Unproc/
	    imr = "Unproc/" // imin
	    copy (temp1, imr)

#  Trim, subtract overscan, and fix bad pixels using mscred.ccdproc
	    ccdproc.bpmasks = ""
	    ccdproc.ccdtype = ""
	    ccdproc.noproc = no
	    ccdproc.xtalkcor = no
	    ccdproc.fixpix = no
	    ccdproc.overscan = no
	    ccdproc.trim = no
	    ccdproc.zerocor = no
	    ccdproc.darkcor = no
	    ccdproc.flatcor = no
	    ccdproc.sflatcor = no
	    ccdproc.split = no
	    ccdproc.merge = no
	    ccdproc.xtalkfile = ""
	    ccdproc.fixfile = "BPM"
	    ccdproc.saturation = "INDEF"
	    ccdproc.sgrow = 0
	    ccdproc.bleed = "INDEF"
	    ccdproc.btrail = 20
	    ccdproc.bgrow = 0
	    ccdproc.biassec = "[1080:1120,3:4610]"
	    ccdproc.trimsec = "!TRIMSEC"
	    ccdproc.zero = "ZERO"
	    ccdproc.dark = "DARK"
	    ccdproc.flat = "FLAT"
	    ccdproc.sflat = ""
	    ccdproc.minreplace = 1.
	    ccdproc.interactive = no
	    ccdproc.function = "legendre"
	    ccdproc.order = 1
	    ccdproc.sample = "*"
	    ccdproc.naverage = 1
	    ccdproc.niterate = 1
	    ccdproc.low_reject = 3.
	    ccdproc.high_reject = 3.
	    ccdproc.grow = 0.
	    ccdproc.fd = ""
	    ccdproc.fd2 = ""
	    temp2 = mktemp (imroot) // "pc" // imx
	    if (verbose)
		print ("HPROC: fixpix, overscan, and trim ", imin)
	    mscred.ccdproc (temp1, output=temp2, fixpix+, overscan+, trim+)

#  Adjust amplifiers to same gain
	    temp3 = mktemp (imroot) // "pcg" // imx
	    if (verbose)
		print ("PROCAMPS: gain correction for ", imin)
	    gaincorr (temp2, temp3)

#  Rename original file
	    imr = "Raw/" // imin
	    rename (imin, imr)

#  Rename result to same as original file
	    rename (temp3, imin)

	    if (deltemp) {
		imdelete (temp1)
		imdelete (temp2)
		}
	    }
	if (deltemp)
	    delete (templist)
end

# May 20 2004	New program to process list of multiextension images
# May 21 2004	Add header line for preamp fix; move original file to Raw/
# Jun  8 2004	Change name of task from PROCAMPS to HPROC
# Jun  8 2004	Save preampfixed, unccdproc'ed files to Unproc/
