# HTRAN -- Correct for fiber area coverage and transmission differences and 
# By Doug Mink, SAO/TDC
# June 10, 2004

procedure htran (input)

string	input = ""	{prompt="Input multispec image to correct"}
string	skycorr=""	{prompt="Transmission from sky flat via maketran"}
bool deltemp = no	{prompt="Delete temporary files (y or n)"}
bool saveold = yes	{prompt="Save unprocessed files to Nogeo/ and Notran/ (y or n)"}
bool verbose = yes	{prompt="Log progress (y or n)"}
bool debug = no		{prompt="Log each aperture (y or n)"}

begin
	bool vb, db
	string	imtype, mstype, keyroot, apxkey, apykey, rratio, aps
	file imin, imout, imcorr, geores, geotemp, imroot, newfile
	string imageap, history, order, function
	int index, iap, nap, npix, lroot, xcol, ycol
	real x, y, rad, ratio

	imin = input
	imcorr = skycorr
	vb = verbose
	db = debug

	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imout = substr (imin, 1, lroot) // ".corr.ms.fits"
	    }
	else {
	    print ("HTRAN: ", imin," is not a multispec file")
	    return
	    }

# Make sure filename ends in .fits
	smatch (imin, ".fits")
	if (!smatch.found) {
	    imin = imin // ".fits"
	    }

	if (vb) {
	    print ("HTRAN: ", imin," -> ",imout)
	    }

# Get number of apertures and pixels from header of input multispec image
	imgetp.verbose = db
	imgetp (imin, "i_naxis1")
	npix = int (imgetp.value)
	imgetp (imin, "i_naxis2")
	nap = int (imgetp.value)
	if (vb) {
	    print ("HTRAN: ", nap," apertures of ",npix," pixels each")
	    }

# Check to see whether geometrical flux correction has been done
	imgetp.verbose = db
	imgetp.default = "0"
	imgetp (imin, "FIBRGEOM")
	if (imgetp.value == "0") {

# Set up temporary geometric correction file
	    geotemp = mktemp ("geocorr") // ".ms.fits"
	    if (vb) {
		print ("HTRAN: radius normalization file is ", geotemp)
		}
	    chpixtype (imin, geotemp, "real")

# Compute geometric correction for each fiber based on its position
	    for (iap = 1; iap <= nap; iap = iap + 1) {
		if (iap == 1) {
		    apxkey = "APID" // iap // ",6"
		    imgetp.default = ""
		    imgetp (imin, apxkey)
		    if (imgetp.value == "") {
			xcol = 3
			ycol = 4
			}
		    else {
			xcol = 6
			ycol = 7
			}
		    }
		apxkey = "APID" // iap // "," //xcol
		imgetp.default = 9999.0
		imgetp (imin, apxkey)
		x = real (imgetp.value)
		apykey = "APID" // iap // "," //ycol
		imgetp (imin, apykey)
		y = real (imgetp.value)
		rad = sqrt (x * x + y * y)
		ratio = 1.11748 + (0.0001866 * rad) - (0.00000188622 * rad * rad);
		rratio = ratio
		imageap = geotemp // "[*," // iap // "]"
		if (db) {
		    print ("HTRAN: ", imin,"[",iap,"]: ", x, y, rad, " => ", ratio)
		    }
		imreplace (imageap, ratio)
		}

# Correct all spectra for optical projection differences
	    geores = mktemp ("geores") // ".ms.fits"
	    imarith (imin, "/", geotemp, geores, verbose=yes)
	    if (deltemp) {
		delete (geotemp)
		}

# Add history line including time about geometry correction
	    gettime ()
	    history = gettime.date // " " // gettime.time // " geometric correction made"
	    hedit (geores, "FIBRGEOM", history, add+, verify-)

# Move original file to Nogeo directory
	    if (saveold) {
		if (!access ("Nogeo/")) {
		    mkdir ("Nogeo")
		    }
		newfile = "Nogeo/" // imin
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		move (imin, "Nogeo")
		}
	    else {
		delete (imin)
		}
	    imrename (geores, imin)
	    }

# Check to see whether fiber color transmission correction has been done
	imgetp.default = "0"
	imgetp (imin, "FIBRTRAN")

# Correct all spectra for fiber color transmission differences
	if (imgetp.value == "0" && imcorr != "") {
	    imarith (imin, "/", imcorr, imout, verbose=yes)

# Move original file to Notran directory
	    if (saveold) {
		if (!access ("Notran/")) {
		    mkdir ("Notran")
		    }
		newfile = "Notran/" // imin
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		move (imin, "Notran")
		}
	    else {
		delete (imin)
		}

# Rename result file to original file name
	    imrename (imout, imin)

# Add history lines including time about this transmission correction file
	    gettime ()
	    imgetp (imcorr, "TRANORD")
	    order = imgetp.value
	    imgetp (imcorr, "TRANFUNC")
	    function = imgetp.value
	    gettime ()
	    history = gettime.date // " " // gettime.time // " fiber transmission corrected: " // order // " order " // function
	    hedit (imin, "FIBRTRAN", history, add+, verify-)
	    hedit (imin, "FIBRFILE", imcorr, add+, verify-)
	    }

end
