# XCWAV1 - Piecewise cross-correlate a spectrum against a template in wavelength
#          space using XCSAO and save the result in a table or the spectrum header
# July 21, 2003
# By Doug Mink, SAO Telescope Data Center

procedure xcwav1 (object, objtemp)

file	object		{prompt="Object spectrum file"}
file	objtemp		{prompt="Object template sectioned file"}

string	mode="pixel"	{prompt="Compare in wavelength or pixel space"}
int	objband=0	{prompt="Object band"}
int	nbin=256	{prompt="Number of points to cross-correlate (rebinned)"}
bool	intxcor=no	{prompt="Run xcsao interactively (y or n)"}
real	ndev=0		{prompt="Reject past this number of standard deviations"}
real	rmin=2.0	{prompt="Reject segment cross-correlations with R < this"}
bool	intdisp=no	{prompt="Fit dispersion function interactively (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

begin
	char arg
	string templist	# File to which to write list of segments
	file xcout	# File to which to write xcsao or emsao output
	file objfile, tempfile, ctempfile, ttempfile, ttempfile1
	string dbfile	# id file created by reref
	string dbfile1	# reref output id file renamed to match correlated file
	int band

	real constant
	real nd
	int im
	bool intact

	if (ndev < 1) {
	    nd = 10
	    }
	else {
	    nd = ndev
	    }
	intact = intdisp
	objfile = object
	band = objband
	templist = objtemp
	ctempfile = caltemp
	ttempfile = calshift

# Cross-correlate spectrum against template, producing output in xcout
	rvsao (motd=no)
	xcout = mktemp (mode)
	if (mode == "xcsao") {
	    arg = "@" // objtemp
	    xcsao (objfile,specband=band,templates=arg,tempband=band,correlate="wavelength",ncols=nbin,report_mode=16,logfile=xcout,curmode=intxcor,debug=debug)
	    if (debug) {
		print ("xcsao output written to ",xcout)
		}
	else {
	    emsao (objfile,specband=band,report_mode=16,logfile=xcout,curmode=intxcor,debug=debug)
	    }

# Read pixels and shifts from xcout and fit a curve to them, then make
# a new database/id file with modified pixel positions for lines
	
	eref (objfile, rmin=rmin, smin=nd, mode=mode, shiftfile=xcout, verbose=verbose, debug=debug)

	delete (xcout)
end

# Jul 21 2003	New task
